#ifndef CoCoA_cocoa4io_H
#define CoCoA_cocoa4io_H

//   Copyright (c)  2007  Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

//#include "CoCoA/library.H"

#include "CoCoA/TmpGTypes.H"  // PolyList and VectorList

#include <string>

namespace CoCoA
{
  class SparsePolyRing; // forward declaration -- CoCoA/SparsePolyRing.H
  class FreeModule; // forward declaration -- CoCoA/FreeModule.H
  class ConstRefRingElem; // forward declaration -- CoCoA/ring.H
  class ZZ; // forward declaration -- CoCoA/ZZ.H
  class PPOrdering; // forward declaration -- CoCoA/PPOrdering.H
  class matrix; // forward declaration -- CoCoA/matrix.H
  //----------------------------------------------------------------------
  enum SkipTagType {TagWasRead, GetTag};  // used in functions Read...(...)

  //----------------------------------------------------------------------
  std::string ReadOperationString(std::istream& in);
  void ReadPolyList(std::istream& in, PolyList& PL, const SparsePolyRing& P, SkipTagType ST);
  void ReadVectorList(std::istream& in, VectorList& L, const FreeModule& FM, SkipTagType ST);

  // cannot be "void Read..(res, ...)" because there is no empty ctor
  // anyway, they are reference counted objects
  matrix ReadIntegerMatrix(std::istream& in, SkipTagType ST);
  matrix ReadRationalMatrix(std::istream& in, SkipTagType ST);
  FreeModule ReadFreeModule(std::istream& in, SkipTagType ST);
  SparsePolyRing ReadPolyRing(std::istream& in, SkipTagType ST);
  PPOrdering ReadPolyRingOrdering(std::istream& in, size_t NumInds, SkipTagType ST);

  //------------------------------
  void PrintTimeToLog(double T);
  void PrintTimeToCoCoA4(double T);
  void PrintVersionToCoCoA4();

  void WritePolyRing(std::ostream& out, const SparsePolyRing& P);
  void WriteMonomials(std::ostream& out, ConstRefRingElem p);
  void WriteMatrixInVar(std::ostream& out, const std::string& VarName, const matrix &M);
  void WritePolyListInVar(std::ostream& out, const std::string& VarName, const PolyList& PL);
  void WriteVectorListInVar(std::ostream& out, const std::string& VarName, const VectorList& VL);
  //----------------------------------------------------------------------
  void AssertTag(const std::string& input_tag, const std::string& expected_tag);
  void SkipTag(std::istream& in, const std::string& expected_tag);
  void ThrowInputError(const std::string& unknown_tag);

  //----------------------------------------------------------------------
  SparsePolyRing NewPolyRingServer(ZZ charact, size_t FloatPrecision, size_t NumParams, const PPOrdering& O);
}


// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/CoCoA4io.H,v 1.2 2007/10/30 17:14:12 abbott Exp $
// $Log: CoCoA4io.H,v $
// Revision 1.2  2007/10/30 17:14:12  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1  2007/04/27 14:54:22  bigatti
// -- content of CoCoAServer.C split into dedicated files
// -- new registration mechanism (through include "RegisterServerOps.H")
//

#endif
