//   Copyright (c)  1997-2006  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License (version 2)
//   as published by the Free Software Foundation.  A copy of the full
//   licence may be found in the file COPYING in this directory.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoA; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef DUPZFACTOR_LIFT_H
#define DUPZFACTOR_LIFT_H

#include "DUPZ.h"
#include "DUPFF.h"
#include "DUPFFlist.h"
#include "DUPI.h"
#include "DUPZfactor_info1.h"

/* The factorization to be lifted is arranged in a "balanced" binary tree */
/* using the struct below.                                                */

struct DUPZfactor_lift_struct;
typedef struct DUPZfactor_lift_struct *DUPZfactor_lifter;

struct DUPZfactor_lift_struct
{
  DUPFF *g, *h;    /* p-adic expansions of the factors, to be found     */
  int r;           /* Expansion of f is up to p^r, of g,h up to p^(r-1) */
  DUPI E;          /* The "error" in component p^r                      */
  DUPFF grecip, hrecip;  /* auxiliary values used during lifting        */
  DUPZfactor_lifter g_lifter, h_lifter;
  DUPI fr;         /* workspace */
  DUPFF Ep;        /* workspace */
};


/***************************************************************************/

DUPZfactor_lifter DUPZfactor_lifter_ctor(DUPFF *g, DUPFF *h, DUPZfactor_lifter g_lifter, DUPZfactor_lifter h_lifter, int rmax);
void DUPZfactor_lift_init(DUPZfactor_info this);
void DUPZfactor_lift_step(DUPZfactor_lifter, DUPFF f_cmpt_r);
int DUPZfactor_lift_output(DUPZ *factors, const DUPZfactor_lifter this, int i);
void DUPZfactor_lift_dtor(DUPZfactor_lifter);

void DUPZfactor_lift_revise(DUPZfactor_info info);
void DUPZfactor_lift_revise1(DUPZfactor_info info, int height);


#endif
